/**
  *  \file client/widgets/racelist.h
  *  \brief Race List widget
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WIDGETS_RACELIST_H
#define PCC_V2_CLIENT_WIDGETS_RACELIST_H

#include "ui/ui.h"
#include "game/playerset.h"

/** List of Race or Team names. Displays (a subrange of) all player
    names, optionally each in his own color. Clicking a name emits a
    signal. Used for score charts and other tables. */
class WRaceList : public UIWidget {
    int preferred_width;          /**< Preferred width of the widget. For FlowLayout. */
    int from;                     /**< First player to display (inclusive). */
    int to;                       /**< Last player to display (inclusive). */
    int current_click;            /**< If mouse is on a player, its number (i.e.\ last emitted signal). */
    int flags;                    /**< Layout selection. */

    GPlayerArray<GfxRect> positions;
    GPlayerSet highlight_races;
    void calcLayout(GPlayerArray<GfxRect>& positions, int w);

 public:
    enum {
        // Layout. Specify one.
        HorizontalLayout = 1,     /**< A single horizontal line. */
        VerticalLayout   = 2,     /**< A vertical column, all lines with same length. */
        FlowLayout       = 4,     /**< Lines wrapped at preferred_width. */

        // Content. Specify one.
        ShowRaceNames    = 8,     /**< Display race adjectives. */
        ShowLetters      = 16,    /**< Display single letters, all same size. */
        ShowTeamNames    = 32,    /**< Display team names. */

        // Colors.
        NoColors         = 64     /**< Do not use colors. All in same color. */
    };

    WRaceList(int preferred_width, int flags, int from, int to);
    ~WRaceList();

    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    bool handleEvent(const UIEvent& e, bool second_pass);

    void setHighlight(GPlayerSet set);
    void setFlags(int i);

    /** Signals a player click.
        \param int player number */
    Signal1<int> sig_click;

    static const uint8 colors[];
};

#endif
