/**
  *  \file client/widgets/taskeditlist.cc
  *  \brief WAutoTaskEditorList widget
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/taskeditlist.h"
#include "int/exec.h"

/** Constructor.
    \param editor Editor reference
    \param lines Preferred height in lines
    \param width Preferred display width in pixels */
WAutoTaskEditorList::WAutoTaskEditorList(const IntAutoTaskEditor& editor, int lines, int width)
    : UIListbox(0, lines, width, 0),
      editor(editor)
{ }

WAutoTaskEditorList::~WAutoTaskEditorList()
{ }

/** Update display after changed auto task.
    \param new_task true if task is completely new, false if it was edited */
void
WAutoTaskEditorList::update(bool new_task)
{
    setSize(editor.getNumInstructions()+1);
    if (new_task) {
        /* Entirely new task */
        setTop(0);
        setCurrent(editor.getNumInstructions());
    }
    adjustPosition();
    drawWidget();
}

void
WAutoTaskEditorList::drawPart(GfxCanvas& can, int from, int to)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useFont(*fonts[FONT_NORMAL]);

    for (int i = from; i < to; ++i) {
        GfxRect area;
        prepareEntry(ctx, ColorStyle, i, &area);

        uint32_t line = getTop() + i;
        int cursorSpace = font_heights[FONT_NORMAL];
        if (line < editor.getNumInstructions()) {
            /* A program line */
            const string_t& cmd = editor[line];
            if (!cmd.empty() && cmd[0] == '%') {
                ctx.setColor(UIColor::tc_Faded);
            }
            outTextF(ctx, area.x + cursorSpace, area.y, area.w - cursorSpace, cmd);
        } else if (line == editor.getNumInstructions()+1) {
            /* After the end */
            ctx.setColor(UIColor::tc_Faded);
            drawHLine(ctx, area.x, area.y, area.x + area.w - 1);
        }
        if (line == editor.getPC()) {
            /* Cursor. PCC1 had these as bitmaps. */
            int y = area.y + font_heights[FONT_NORMAL]/2 - 1;
            int steps = 5;
            int xl = area.x + 3;
            ctx.setColor(UIColor::tc_Red);
            if (editor.isInSubroutineCall()) {
                /* Hollow triangle */
                for (int dy = 0; dy < steps; ++dy) {
                    int y1 = y + dy;
                    int y2 = y - dy;
                    drawPixel(ctx, GfxPoint(xl, y1));
                    drawPixel(ctx, GfxPoint(xl + 2*(steps-dy) - 1, y1));
                    drawPixel(ctx, GfxPoint(xl + 2*(steps-dy) - 2, y1));
                    if (y1 != y2) {
                        drawPixel(ctx, GfxPoint(xl, y2));
                        drawPixel(ctx, GfxPoint(xl + 2*(steps-dy) - 1, y2));
                        drawPixel(ctx, GfxPoint(xl + 2*(steps-dy) - 2, y2));
                    }
                }
            } else {
                /* Solid triangle */
                for (int dy = 0; dy < steps; ++dy) {
                    int y1 = y + dy;
                    int y2 = y - dy;
                    drawHLine(ctx, xl, y1, xl + 2*(steps-dy) - 1);
                    if (y1 != y2) {
                        drawHLine(ctx, xl, y2, xl + 2*(steps-dy) - 1);
                    }
                }
            }
        }
    }
}
