/**
  *  \file game/storage.cc
  *  \brief Game Data Storage
  *
  *  This module merges the knowledge of loading and saving a game in
  *  a class, so people can save a game without having to know how it
  *  was loaded.
  *
  *  \todo The interface is still a bit whacky. In particular, these
  *  classes assume that game_file_dir is already set. Is this good?
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#include "game/storage.h"
#include "game/load.h"
#include "game/save.h"
#include "game/gamedirs.h"
#include "cpluslib/format.h"
#include "game/utilparse.h"
#include "game/global.h"
#include "game/fleet.h"

Ptr<GGameDataStorage> GGameDataStorage::instance;

/**************************** GGameDataStorage ***************************/

GGameDataStorage::~GGameDataStorage()
{ }

/************************* GGameDirectoryStorage *************************/

GGameDirectoryStorage::GGameDirectoryStorage(int player)
    : player(player)
{ }

void
GGameDirectoryStorage::load(GGameTurn& trn)
{
    loadDirectory(trn, *game_file_dir, player);
    loadFleets(trn, *game_file_dir, player);
}

void
GGameDirectoryStorage::save(GGameTurn& trn)
{
    saveDirectory(trn, *game_file_dir, GPlayerSet(player));
    saveFleets(trn, *game_file_dir, player);
}

/*************************** GGameResultStorage **************************/

GGameResultStorage::GGameResultStorage(int player, bool have_trn)
    : player(player),
      have_trn(have_trn)
{ }

void
GGameResultStorage::load(GGameTurn& trn)
{
    Ptr<Stream> rst = game_file_dir->openFile(format("player%d.rst", player), Stream::C_READ);
    loadResult(trn, *rst, player, true);
    if (have_trn) {
        Ptr<Stream> trnfile = game_file_dir->openFile(format("player%d.trn", player), Stream::C_READ);
        loadTurn(trn, *trnfile, player);
    }

    loadFleets(trn, *game_file_dir, player);

    /* Util */
    /* FIXME: it is ugly to do this here; loadDirectory does it itself */
    if (Ptr<Stream> s = game_file_dir->openFileNT(format("util%d.dat", player), Stream::C_READ))
        (GUtilParser(trn, player)).read(*s);

    /* FLAK */
    maybeLoadFlakVcrs(trn, *game_file_dir, player);
}

void
GGameResultStorage::save(GGameTurn& trn)
{
    saveTurns(trn, *game_file_dir, GPlayerSet(player));
    saveFleets(trn, *game_file_dir, player);
}
