/**
  *  \file game/timestamp.h
  *  \brief Timestamps
  *
  *  This module contains functions to operate on timestamps.
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_TIMESTAMP_H
#define PCC_V2_GAME_TIMESTAMP_H

#include <cstring>
#include "cpluslib/string.h"

//! Timestamp.
class GTimestamp {
 public:
    enum { SIZE = 18 };
#ifdef __BORLANDC__
    typedef char* data_t;
    typedef const char* const_data_t;
#else
    typedef char (&data_t)[SIZE];
    typedef const char (&const_data_t)[SIZE];
#endif
 private:
    char data[SIZE];
 public:
    /** Construct from timestamp data field. */
    GTimestamp(const_data_t data)
        { std::memcpy(this->data, data, SIZE); }
    GTimestamp();

    string_t getTimestampAsString() const;
    string_t getTimeAsString() const;
    string_t getDateAsString() const;

    /** Get raw data. */
    const_data_t getRawData() const
        { return data; }
    void storeRawData(data_t data) const;

    /** Compare for equality. */
    bool operator==(const GTimestamp& rhs) const
        { return std::memcmp(data, rhs.data, SIZE) == 0; }
    bool operator==(const_data_t rhs) const
        { return std::memcmp(data, rhs, SIZE) == 0; }
    /** Compare for inequality. */
    bool operator!=(const GTimestamp& rhs) const
        { return std::memcmp(data, rhs.data, SIZE) != 0; }
    bool operator!=(const_data_t rhs) const
        { return std::memcmp(data, rhs, SIZE) != 0; }

    bool isEarlierThan(const GTimestamp& rhs) const;

    void copyFrom(const char* p)
        { std::memcpy(this->data, p, SIZE); }
};

#endif
