/**
  *  \file resmgr/resdir.h
  *  \brief ResDirectory resource provider class
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_RESMGR_RESDIR_H
#define PCC_V2_RESMGR_RESDIR_H

#include "resmgr/resmgr.h"

/** Resources from a directory (dir:NAME). This provides resources
    from files in a directory. File names are either synthesized from
    the ResId, or taken from an alias table. The alias table can refer
    to file names, and build synthetic resources. */
class ResDirectory : public ResProvider {
    /// Alias table entry.
    struct Node {
        string_t name;
    };

    /// Type for alias definitions.
    typedef std::map<string_t, Node> alias_map_t;

    /// Alias definitions.
    alias_map_t alias_map;

    /// Underlying directory.
    Ptr<Directory> dir;

    void loadAliases(Stream& s);
    Ptr<Stream> openStream(string_t name, const char*const* suffixes);
 public:
    ResDirectory(Ptr<Directory> dir);
    ~ResDirectory();

    Ptr<GfxPixmap> loadPixmap(ResId id);
    SfxSampleData* loadSound(ResId id);
    Ptr<GfxAnimation> loadAnimation(ResId id);
    Ptr<GfxFont> getFont(ResId id);
};

#endif
